;*****************************************************************************
;
; Subroutine to debounce up to 8 switches
;
; Program	DEBOUNCE.INC
; Author	Wim de Vree, pe1grl@planet.nl
;
; Revision history
;	20030714	Copied from Scott Dattalo & Terje Mathisen, see www.dattalo.com
;
; Usage
; 	See explanation below, routine requires 4 variables;
;	You will have to repeately call the Debounce routine
;	(e.g. from a timer interrupt)
;	
; Example
;		DBNCE_VAR	equ 0x20	; Used registers, 4 required
;
;		movf	SWITCHES,W
;		movwf	W,DBNCE_CSA		; Set switch status
;		call 	Debounce		; Filtered status is in DBNCE_CVA
;	
;
; The purpose of this routine is to debounce, i.e. digitally low pass filter
; inputs. The algorithm handles upto 8 bits at a time. An input is considered
; filtered if it has not changed states in the last 4 samples.
;
; 2-bit cyclic vertical counters count the 4 samples. As long as there is no
; change, the counters are held in the reset state of 00b. When a change is detected
; between the current sample and the filtered or debounced sample, the counters
; are incremented. The counting sequence is 00,01,10,11,00... When the counters
; roll over from 11b to 00b, the debounced state is updated. If the input changes
; back to the filtered state while the counters are counting, then the counters
; are re-initialized to the reset state and the filtered state is unaffected.
; In other words, a glitch or transient input has been filtered.
;
; Here's the C-psuedo code:
;
; static unsigned clock_A,clock_B,debounced_state;
; debounce(unsigned new_sample)
; {
;   unsigned delta;
;
;   delta = new_sample ^ debounced_state;   //Find all of the changes
;
;   clock_A ^= clock_B;                     //Increment the counters
;   clock_B  = ~clock_B;
;
;   clock_A &= delta;                       //Reset the counters if no changes
;   clock_B &= delta;                       //were detected.
;
;       //Preserve the state of those bits that are being filtered and simultaneously
;       //clear the states of those bits that are already filtered.
;   debounced_state &= (clock_A | clock_B);
;       //Re-write the bits that are already filtered.
;  debounced_state |= (~(clock_A | clock_B) & new_sample);
; }
;
; The 2-bit counters are arranged "vertically". In other words 8 counters
; are formed with 2 bytes such that the corresponding bits in the bytes are
; paired (e.g. MSBit of each byte is paired to form one counter).
; The counting sequence is 0,1,2,3,0,1,... And the state tables and Karnaugh
; maps are:
;
; State Table:     Karnaugh Maps:
; pres  next      B
;  SS  SS         0   1
;  AB  AB       +---+---+    +---+---+
; --------   A 0|   | 1 |    | 1 |   |
;  00  01       +---+---+    +---+---+
;  01  10      1| 1 |   |    | 1 |   |
;  10  11       +---+---+    +---+---+
;  11  00      A+ = A ^ B     B+ = ~B
; 
;  Here's the PIC code that implements the counter:
;        MOVF    SB,W    ;W = B
;        XORWF   SA,F    ;A+ = A ^ B
;        COMF    SB,F    ;B+ = ~B

; 14 instructions
; 15 cycles
; Inputs:
;   csa - The current sample
; Outputs
;   cva - The current value (filtered version of csa)
;
; RAM used
;   count_A,
;   count_B - State variables for the 8 2-bit counters
;

#define	DBNC_cva	(DBNC_VAR)
#define	DBNC_csa	(DBNC_VAR+1)
#define	DBNC_countA	(DBNC_VAR+2)
#define	DBNC_countB	(DBNC_VAR+3)

DeBounce:	MOVF    DBNC_countB,W		; Increment the vertical counter
        	XORWF   DBNC_countA,F
        	COMF    DBNC_countB,F
        	MOVF    DBNC_csa,W		; See if any changes occurred
        	XORWF   DBNC_cva,W
        	ANDWF   DBNC_countB,F		; Reset the counter if no change has occurred
        	ANDWF   DBNC_countA,F
        	MOVF    DBNC_countB,W		; Determine the counter's state
        	IORWF   DBNC_countA,W
    				
       		ANDWF   DBNC_cva,F		; Clear all bits that are filtered-or more accurately,
        	XORLW   0xff    		; save the state of those that are being filtered

        	ANDWF   DBNC_csa,W		; Re-write the bits that haven't changed.
        	IORWF   DBNC_cva,F
        	RETURN
