;	PIC16 24 BIT FLOATING POINT LIBRARY	VERSION 1.14
;
;	Unary operations: both input and output are in AEXP,AARG
;	Binary operations: input in AEXP,AARG and BEXP,BARG with output in AEXP,AARG
;	All routines return WREG = 0x00 for successful completion, and WREG = 0xFF
;	for an error condition specified in FPFLAGS.
;
;	All timings are worst case cycle counts
;         Routine	  	Function
;	FLO24	  	16 bit integer to 24 bit floating point conversion
;
;		Timing:		   RND
;				0	1
;			0	83	83
;		   SAT
;			1	88	88
;	NRM24	  	normalization of unnormalized 24 bit floating point numbers
;		Timing:		   RND
;				0	1
;			0	72	72
;		   SAT
;			1	77	77
;
;	INT24	  	24 bit floating point to 16 bit integer conversion
;		Timing:		   RND
;				0	1
;			0	80	86
;		   SAT
;			1	80	89
;
;	FPA24	 	24 bit floating point add
;		Timing:		   RND
;				0	1
;			0	194	205
;		   SAT
;			1	194	210
;
;	FPS24	 	24 bit floating point subtract
;		Timing:		   RND
;				0	1
;			0	196	207
;		   SAT
;			1	196	212
;
;	FPM24	 	24 bit floating point multiply
;		Timing:		   RND
;				0	1
;			0	298	309
;		   SAT
;			1	298	313
;
;	FPD24	 	24 bit floating point divide
;		Timing:		   RND
;			0	1
;			0	469	491
;		   SAT
;			1	469	495
;
; 27jul2003	Fixed a bug in FPADD24. Note: there is a newer (but bigger)
;		version available from MicroChip

;     define assembler constants
B0		equ	0
B1		equ	1
B2		equ	2
B3		equ	3
B4		equ	4
B5		equ	5

MSB		equ	7
LSB		equ	0

;**********************************************************************************************
;	24 bit floating point representation
;
;	EXPONENT	8 bit biased exponent
;			It is important to note that the use of biased exponents produces
;			a unique representation of a floating point 0, given by
;			EXP = HIGHBYTE = LOWBYTE = 0x00, with 0 being the only
;			number with EXP = 0.
;
;	HIGHBYTE	8 bit most significant byte of sign-magnitude representation, with
;			SIGN = MSB, and implicit mantissa MSB = 1 and radix point to the
;			left of MSB
;
;	LOWBYTE	8 bit least significant byte of sign-magnitude matissa
;
;			RADIX
;	EXPONENT	POINT 	HIGHBYTE	LOWBYTE
;
;	xxxxxxxx           .	Sxxxxxxx	xxxxxxxx


;	Define literal constants
	constant EXPBIAS = .128

;	Define FP library register variables 12 bytes required
	
EXP 	equ FPVAR	; 8 bit biased exponent
ACC 	equ FPVAR+.1	; most significant byte of contiguous 4 byte accumulator
SIGN	equ FPVAR+.5	; save location for sign in MSB
FPFLAGS	equ FPVAR+.6	; floating point library exception flags
BEXP	equ FPVAR+.7	; 8 bit biased exponent for argument B
BARG	equ FPVAR+.8	; most significant byte of mantissa for argument B
FPTEMP	equ FPVAR+.10	; temporary storage, 2 bytes required

;	Note:	(AEXP, AARG+B0, AARG+B1 )  and  (EXP, ACC+B0, ACC+B1)
AEXP		equ	EXP	; 8 bit biased exponent for argument A
AARG		equ	ACC	; most significant byte of mantissa for argument A

FPLDC24	MACRO	VALUE				; Load a 16 bit constant in the FP accumulator
	movlw	(VALUE / .256)
	movwf	AARG
	movlw	(VALUE % .256)
	movwf	AARG+1
	call	FPFLO24
	ENDM

FPLDI24	MACRO	ARG				; Load a 16 Bit variable in teh FP Accumulator
	movfw	ARG				; and convert to a float
	movwf	AARG+1				; note the byte swap (so shorts are low-high order!)
	movfw	ARG+1
	movwf	AARG
	call	FPFLO24
	ENDM

FPSTI24	MACRO	ARG				; Convert the FP Accumulator to short and get it to
	call	FPINT24				; a user variable
	movfw	AARG+1				; note the byte swap (so shorts are low-high order!)
	movwf	ARG
	movfw	AARG
	movwf	ARG+1
	ENDM

FPLDF24	MACRO	ARG
	movfw	ARG
	movwf	AEXP
	movfw	ARG+1
	movwf	AARG
	movfw	ARG+2
	movwf	AARG+1
	ENDM

FPSTF24	MACRO	ARG
	movfw	AEXP
	movwf	ARG
	movfw	AARG
	movwf	ARG+1
	movfw	AARG+1
	movwf	ARG+2
	ENDM

;	Define foating point library exception flags
IOV		equ	0	; bit0 = integer overflow flag
FOV		equ	1	; bit1 = floating point overflow flag
FUN		equ	2	; bit2 = floating point underflow flag
FDZ		equ	3	; bit3 = floating point divide by zero flag
RND		equ	6	; bit6 = floating point rounding flag, 0 = truncation
				; 1 = rounding to nearest LSB
SAT		equ	7	; bit7 = floating point saturate flag, 0 = terminate on
				; exception without saturation, 1 = terminate on
				; exception with saturation to appropriate value

;**********************************************************************************************
;	Integer to float conversion
;	Input:	16 bit 2's complement integer right justified in AARG+B0, AARG+B1
;	Use:	CALL	FLO24
;	Output:	24 bit floating point number in AEXP, AARG+B0, AARG+B1
;	Result:	AARG  <--  FLOAT( AARG )
;	Max Timing:	11+72 = 83 clks		SAT = 0
;			11+77 = 88 clks		SAT = 1
;	Min Timing:	 7+14 = 21 clks		AARG = 0
;			 7+18 = 25 clks
;	PM: 11+26 = 37			DM: 6

FPFLO24		MOVLW	.16+EXPBIAS	; initialize exponent and add bias
		MOVWF	EXP
		MOVFW	ACC+B0
		MOVWF	SIGN
		BTFSS	ACC+B0,MSB	; test sign
		GOTO	NRM24
		COMF	ACC+B1,f	; if < 0, negate and set MSB in SIGN
		COMF	ACC+B0,f
		INCF	ACC+B1,f
		skpnz
		 INCF	ACC+B0,f

;-------------------------------------------------------------------
;	Normalization routine
;	Input:	24 bit unnormalized floating point number in AEXP, AARG+B0, AARG+B1,
;		with sign in SIGN,MSB and other bits zero.
;	Use:	CALL	NRM24
;	Output:	24 bit normalized floating point number in AEXP, AARG+B0, AARG+B1
;	Result:	AARG  <--  NORMALIZE( AARG )
;	Max Timing:	10+6+7*7+7 = 72 clks		SAT = 0
;					10+6+7*7+1+11 = 77 clks	SAT = 1
;	Min Timing: 14 clks			AARG = 0
;					5+9+4 = 18 clks
;	PM: 26					DM: 6

NRM24		CLRF	FPTEMP		; clear exponent decrement
		MOVF	ACC+B0,W	; test if highbyte=0
		bnz	NORM24
		MOVF	ACC+B1,W	; if so, shift 8 bits by move
		MOVWF	ACC+B0
		bz	RES024		; if highbyte=0, result=0
		CLRF	ACC+B1
		BSF	FPTEMP,3

NORM24		MOVF	FPTEMP,W
		SUBWF	EXP,f
		skpz
		skpc
		GOTO	SETFUN24

		clrc			; clear carry bit

NORM24A		BTFSC	ACC+B0,MSB	; if MSB=1, normalization done
		 GOTO	FIXSIGN24
		RLF	ACC+B1,f	; otherwise, shift left and 
		RLF	ACC+B0,f	; decrement EXP
		DECFSZ	EXP,f
		 GOTO	NORM24A

		GOTO	SETFUN24	; underflow if EXP=0

FIXSIGN24	BTFSS	SIGN,MSB
		BCF	ACC+B0,MSB	; clear explicit MSB if positive
		RETLW	0

;**********************************************************************************************
;	Float to integer conversion
;	Input:	24 bit floating point number in AEXP, AARG+B0, AARG+B1
;	Use:	CALL	INT24
;	Output:	16 bit 2's complement integer right justified in AARG+B0, AARG+B1
;	Result:	AARG  <--  INT( AARG )
;	Max Timing:	26+6*6+5+13 = 80 clks		RND = 0
;			26+6*6+5+19 = 86 clks		RND = 1, SAT = 0
;			26+6*6+5+22 = 89 clks		RND = 1, SAT = 1
;	Min Timing: 9+5 = 14 clks		AARG = 0
;					19+5+7 = 31 clks
;	PM: 64					DM: 6

FPINT24		MOVF	ACC+B0,W	; save sign in SIGN
		MOVWF	SIGN
		BSF	ACC+B0,MSB	; make MSB explicit

		MOVLW	EXPBIAS		; remove bias from EXP
		SUBWF	EXP,f
		BTFSS	EXP,MSB		; if <= 0, result=0
		skpnz
		GOTO	RES024

		MOVF	EXP,W
		SUBLW	.16
		MOVWF	EXP
		skpz
		BTFSC	EXP,MSB
		GOTO	SETIOV24	

		MOVLW	.8		; do byte shift if EXP >= 8
		SUBWF	EXP,W
		bnc	SHIFT24
		MOVWF	EXP
		RLF	ACC+B1,f	; rotate next bit for rounding
		MOVF	ACC+B0,W
		MOVWF	ACC+B1
		CLRF	ACC+B0

		MOVF	EXP,W		; shift completed if EXP = 0
		bz	SHIFT24OK

SHIFT24		CLRC
		RRF	ACC+B0,f	; right shift by EXP
		RRF	ACC+B1,f
		DECFSZ	EXP,f
		 GOTO	SHIFT24

SHIFT24OK	BTFSC	FPFLAGS,RND
		BTFSS	ACC+B1,LSB
		GOTO	INT24OK
		CLRF	ACC+B2
		RLF	ACC+B2,W
		ADDWF	ACC+B1,f
		skpnz
		 INCF	ACC+B0,f
		BTFSC	ACC+B0,MSB	; test for overflow
		GOTO	SETIOV24

INT24OK		BTFSS	SIGN,MSB	; if sign bit set, negate		
		RETLW	0
		COMF	ACC+B1,f
		COMF	ACC+B0,f
		INCF	ACC+B1,f
		skpnz
		 INCF	ACC+B0,f
		RETLW	0

RES024		CLRF	ACC+B0		; integer result equals zero
		CLRF	ACC+B1
		CLRF	EXP		; clear EXP for other routines
		RETLW	0

SETIOV24	BSF	FPFLAGS,IOV	; set integer overflow flag
		BTFSS	FPFLAGS,SAT	; test for saturation
		RETLW	0xFF		; return error code in WREG

		CLRF	ACC+B0		; saturate to largest two's
		BTFSS	SIGN,MSB	; complement 16 bit integer
		MOVLW	0xFF
		MOVWF	ACC+B0		; SIGN = 0, 0x 7F FF
		MOVWF	ACC+B1		; SIGN = 1, 0x 80 00
		RLF	SIGN,f
		RRF	ACC+B0,f
		RETLW	0xFF		; return error code in WREG


;**********************************************************************************************
;	Floating Point Multiply
;	Input:	24 bit floating point number in AEXP, AARG+B0, AARG+B1
;		24 bit floating point number in BEXP, BARG+B0, BARG+B1
;	Use:	CALL	FPM24
;	Output:	24 bit floating point product in AEXP, AARG+B0, AARG+B1
;	Result:	AARG  <--  AARG * BARG
;	Max Timing:	25+15*16+15+18 = 298 clks	RND = 0
;			25+15*16+15+29 = 309 clks	RND = 1, SAT = 0
;			25+15*16+15+33 = 313 clks	RND = 1, SAT = 1
;	Min Timing:	6+5 = 11 clks		AARG * BARG = 0
;			24+15*11+14+15 = 218 clks
;	PM: 80					DM: 11

FPMUL24		MOVF	AEXP,W		; test for zero arguments
		skpz
		 MOVF	BEXP,W
		bz	RES024

M24BNE0		MOVF	AARG+B0,W
		XORWF	BARG+B0,W
		MOVWF	SIGN		; save sign in SIGN

		MOVFW	BEXP
		ADDWF	EXP,f
		MOVLW	EXPBIAS
		bnc	MTUN24

		ADDWF	EXP,f
		bc	SETFOV24	; set multiply overflow flag
		GOTO	MOK24

MTUN24		ADDWF	EXP,f
		bnc	SETFUN24

MOK24		BSF	AARG+B0,MSB	; make argument MSB's explicit
		BSF	BARG+B0,MSB
		clrc
		CLRF   	ACC+B2		; clear initial partial product
		CLRF   	ACC+B3
		MOVLW	.16
		MOVWF	FPTEMP		; initialize counter

MLOOP24		BTFSS	AARG+B1,LSB	; test high byte
		GOTO	MNOADD24

MADD24		MOVF	BARG+B1,W
		ADDWF	ACC+B3,f
		MOVF	BARG+B0,W
		skpnc
		 INCFSZ	BARG+B0,W
		ADDWF	ACC+B2,f

MNOADD24	RRF	ACC+B2,f
		RRF 	ACC+B3,f
		RRF	ACC+B0,f
		RRF	ACC+B1,f
		clrc
		DECFSZ	FPTEMP,f
		 GOTO	MLOOP24

		BTFSC	ACC+B2,MSB	; check for postnormalization
	 	 GOTO	MROUND24
		RLF	ACC+B0,f
		RLF	ACC+B3,f
		RLF	ACC+B2,f
		DECF	EXP,f

MROUND24	BTFSC	FPFLAGS,RND
		BTFSS	ACC+B3,LSB
		GOTO	MUL24OK
		RLF	ACC+B0,f	; rotate next significant bit into
		skpnc
		 INCF	ACC+B3,f	; carry for rounding
		skpnz
		 INCF	ACC+B2,f

		bnz	MUL24OK		; has rounding caused carryout?
		RRF	ACC+B2,f	; if so, right shift
		RRF	ACC+B3,f
		INCF	EXP,f
		bc	SETFOV24	; check for overflow

MUL24OK		BTFSS	SIGN,MSB
		BCF	ACC+B2,MSB	; clear explicit MSB if positive

		MOVF	ACC+B2,W
		MOVWF	AARG+B0		; move result to AARG
		MOVF	ACC+B3,W
		MOVWF	AARG+B1
		RETLW	0  

SETFOV24	BSF	FPFLAGS,FOV	; set floating point underflag
		BTFSS	FPFLAGS,SAT	; test for saturation
		 RETLW	0xFF		; return error code in WREG

		MOVLW	0xFF
		MOVWF	AEXP		; saturate to largest floating
		MOVWF	AARG+B0		; point number = 0x FF 7F FF
		MOVWF	AARG+B1		; modulo the appropriate sign bit
		RLF	SIGN,f
		RRF	AARG+B0,f
		RETLW	0xFF		; return error code in WREG

;**********************************************************************************************
;	Floating Point Divide
;	Input:	24 bit floating point dividend in AEXP, AARG+B0, AARG+B1
;		24 bit floating point divisor in BEXP, BARG+B0, BARG+B1
;	Use:	CALL	FPD24
;	Output:	24 bit floating point quotient in AEXP, AARG+B0, AARG+B1
;	Result:	AARG  <--  AARG / BARG
;	Max Timing:	29+13+15*26+25+12 = 469 clks	RND = 0
;					29+13+15*26+25+34 = 491 clks	RND = 1, SAT = 0
;					29+13+15*26+25+38 = 495 clks	RND = 1, SAT = 1
;	Min Timing:	24+12+15*20+12 = 348 clks
;	PM: 117					DM: 11

FPDIV24		MOVF	BEXP,W		; test for divide by zero
		bz	SETFDZ24

D24BNE0		MOVF	AARG+B0,W
		XORWF	BARG+B0,W
		MOVWF	SIGN		; save sign in SIGN
		BSF	AARG+B0,MSB	; make argument MSB's explicit
		BSF	BARG+B0,MSB

TALIGN24	CLRF	FPTEMP		; clear align increment
		MOVF	AARG+B0,W
		MOVWF	ACC+B2		; test for alignment
		MOVF	AARG+B1,W
		MOVWF	ACC+B3

		MOVF	BARG+B1,W
		SUBWF	ACC+B3,f
		MOVF	BARG+B0,W
		skpc
		 INCFSZ	BARG+B0,W
		SUBWF	ACC+B2,f

		CLRF	ACC+B2
		CLRF	ACC+B3

		bnc	DALIGN24OK

		clrc			; align if necessary
		RRF	(ACC+B0),f
		RRF	(ACC+B1),f
		RRF	(ACC+B2),f
		MOVLW	0x01
		MOVWF	FPTEMP		; save align increment		

DALIGN24OK	MOVF	BEXP,W		; compare AEXP and BEXP
		SUBWF	EXP,f
		bnc	ALTB24
	
AGEB24		MOVLW	EXPBIAS
		ADDWF	FPTEMP,W
		ADDWF	EXP,f
		bc	SETFOV24
		GOTO	DARGOK24	; set overflow flag

ALTB24		MOVLW	EXPBIAS
		ADDWF	FPTEMP,W
		ADDWF	EXP,f
		bnc	SETFUN24	; set underflow flag

DARGOK24	MOVLW	.16		; initialize counter
		MOVWF	FPTEMP+B1

DLOOP24		RLF	ACC+B3,f	; left shift
		RLF	ACC+B2,f
		RLF	ACC+B1,f
		RLF	ACC+B0,f
		RLF	FPTEMP,f

		MOVFW	BARG+B1		; subtract
		SUBWF	ACC+B1,f
		MOVF	BARG+B0,W
		skpc
		 INCFSZ	BARG+B0,W
		SUBWF	ACC+B0,f

		RLF	BARG+B0,W
		IORWF	FPTEMP,f
		BTFSS	FPTEMP,LSB	; test for restore
		GOTO	DREST24

		BSF	ACC+B3,LSB
		GOTO	DOK24

DREST24		MOVFW	(BARG+B1)	; restore if necessary
		ADDWF	ACC+B1,f
		MOVFW	(BARG+B0)
		skpnc
		 INCF	(BARG+B0),W
		ADDWF	(ACC+B0),f
		BCF	(ACC+B3),LSB

DOK24		DECFSZ	FPTEMP+B1,f
		 GOTO	DLOOP24

DROUND24	BTFSC	FPFLAGS,RND
		BTFSS	ACC+B3,LSB
		GOTO	DIV24OK
		clrc
		RLF	ACC+B1,f	; compute next significant bit
		RLF	ACC+B0,f	; for rounding
		RLF	FPTEMP,f

		MOVFW	BARG+B1		; subtract
		SUBWF	ACC+B1,f
		MOVFW	BARG+B0
		skpc
		 INCFSZ	BARG+B0,W
		SUBWF	ACC+B0,f

		RLF	BARG+B0,W
		IORWF	FPTEMP,W
		ANDLW	0x01		

		ADDWF	ACC+B3,f
		skpnc
		 INCF	ACC+B2,f

		bnz	DIV24OK		; test if rounding caused carryout
		RRF	ACC+B2,f
		RRF	ACC+B3,f
		INCF	EXP,f
		bz	SETFOV24	; test for overflow

DIV24OK		BTFSS	SIGN,MSB
		BCF	ACC+B2,MSB	; clear explicit MSB if positive

		MOVFW	ACC+B2
		MOVWF	AARG+B0		; move result to AARG
		MOVFW	ACC+B3
		MOVWF	AARG+B1

		RETLW	0

SETFUN24	BSF	FPFLAGS,FUN	; set floating point underflag
		BTFSS	FPFLAGS,SAT	; test for saturation
		RETLW	0xFF		; return error code in WREG

		MOVLW	0x01		; saturate to smallest floating
		MOVWF	AEXP		; point number = 0x 01 00 00
		CLRF	AARG+B0		; modulo the appropriate sign bit
		CLRF	AARG+B1
		RLF	SIGN,f
		RRF	AARG+B0,f
		RETLW	0xFF		; return error code in WREG

SETFDZ24	BSF	FPFLAGS,FDZ	; set divide by zero flag
		RETLW	0xFF

;**********************************************************************************************
;	Floating Point Subtract
;	Input:	24 bit floating point number in AEXP, AARG+B0, AARG+B1
;		24 bit floating point number in BEXP, BARG+B0, BARG+B1
;	Use:	CALL FPS24
;	Output:	24 bit floating point sum in AEXP, AARG+B0, AARG+B1
;	Result:	AARG  <--  AARG - BARG
;	Max Timing:	2+194 = 196 clks		RND = 0
;					2+205 = 207 clks		RND = 1, SAT = 0
;					2+210 = 212 clks		RND = 1, SAT = 1
;	Min Timing:	2+44 = 46 clks
;	PM: 2+102 = 104		DM: 11

FPSUB24		MOVLW	0x80
		XORWF	BARG+B0,f

;-------------------------------------------------------------------
;	Floating Point Add
;	Input:	24 bit floating point number in AEXP, AARG+B0, AARG+B1
;		24 bit floating point number in BEXP, BARG+B0, BARG+B1
;	Use:	CALL FPA24
;	Output:	24 bit floating point sum in AEXP, AARG+B0, AARG+B1
;	Result:	AARG  <--  AARG - BARG
;	Max Timing:	25+25+6*6+5+31+72 = 194 clks	RND = 0
;					25+25+6*6+5+42+72 = 205 clks	RND = 1, SAT = 0
;					25+25+6*6+5+42+77 = 210 clks	RND = 1, SAT = 1
;	Min Timing:	22+18+4 = 44 clks
;	PM: 102		DM: 11

FPADD24		MOVFW	AARG+B0		; exclusive or of signs in TEMP
		XORWF	BARG+B0,W
		MOVWF	FPTEMP

		MOVFW	AEXP		; use AARG if AEXP >= BEXP
		SUBWF	BEXP,W
		bnc	USEA24

		MOVFW	BEXP
		MOVWF	ACC+B2		; otherwise, swap AARG and BARG
		MOVFW	AEXP
		MOVWF	BEXP
		MOVFW	ACC+B2
		MOVWF	AEXP

		MOVFW	BARG+B0
		MOVWF	ACC+B2
		MOVFW	AARG+B0
		MOVWF	BARG+B0
		MOVFW	ACC+B2
		MOVWF	AARG+B0

		MOVFW	BARG+B1
		MOVWF	ACC+B2
		MOVFW	AARG+B1
		MOVWF	BARG+B1
		MOVFW	ACC+B2
		MOVWF	AARG+B1

USEA24		MOVF	AARG+B0,W
		MOVWF	SIGN		; save sign in SIGN
		BSF	AARG+B0,MSB	; make MSB's explicit
		BSF	BARG+B0,MSB

		MOVF	BARG+B0,W
		MOVWF	ACC+B2
		MOVF	BARG+B1,W
		MOVWF	ACC+B3

		MOVF	BEXP,W		; compute shift count in BEXP
		SUBWF	AEXP,W
		MOVWF	BEXP
		bz	AROUND24

		MOVLW	.8
		SUBWF	BEXP,W
		bnc	ALIGNB24	; if BEXP >= 8, do byte shift
		MOVWF	BEXP
		RLF	ACC+B3,f	; rotate next bit for rounding
		MOVF	ACC+B2,W
		MOVWF	ACC+B3
		CLRF	ACC+B2

ALIGNB24	MOVF	BEXP,W		; already aligned if BEXP = 0
		bz	AROUND24

ALOOPB24	clrc			; right shift by BEXP
		RRF	ACC+B2,f
		RRF	ACC+B3,f
		DECFSZ	BEXP,f
		 GOTO	ALOOPB24

AROUND24	BTFSC	FPFLAGS,RND
		BTFSS	ACC+B3,LSB
		GOTO	ALIGNED24
		skpc
		 GOTO	ALIGNED24
		INCF	ACC+B3,f	; carry for rounding
		skpnz
		 INCF	ACC+B2,f
		skpz
		 GOTO	ALIGNED24
		RRF	ACC+B2,f
		RRF	ACC+B3,f
		INCF	EXP,f
		skpnz
		 GOTO	SETFOV24

ALIGNED24	BTFSS	FPTEMP,MSB	; negate if signs opposite
		 GOTO	AOK24
		COMF	ACC+B3,f
		COMF	ACC+B2,f
		INCF	ACC+B3,f
		skpnz
		 INCF	ACC+B2,f

;AOK24		MOVFW	ACC+B3		; add
;		ADDWF	AARG+B1,f
;		skpnc
;		 INCF	ACC+B0,f
;		MOVFW	ACC+B2
;		ADDWF	AARG+B0,f

; Wimpie Bugfixed version
AOK24		MOVFW	ACC+B3		; add
		ADDWF	AARG+B1,f
		movfw	ACC+B2
		skpnc
		 INCFSZ	ACC+B2,w
		ADDWF	AARG+B0,f

		BTFSC	FPTEMP,MSB
		 GOTO	ACOMP24
		skpc
		 GOTO	FIXSIGN24

		RRF	AARG+B0,f	; shift right and increment EXP
		RRF	AARG+B1,f
		INCFSZ	AEXP,f
		 GOTO	FIXSIGN24
		GOTO	SETFOV24

ACOMP24		skpnc
		 GOTO	NRM24		; normalize and fix sign

		COMF	AARG+B1,f	; negate, toggle sign bit and
		COMF	AARG+B0,f	; then normalize
		INCF	AARG+B1,f
		skpnz
		 INCF	AARG+B0,f

		MOVLW	0x80
		XORWF	SIGN,f
		GOTO	NRM24
		
;**********************************************************************************************
;	Floating Point Swap
;
;	Swap contents of A and B register
;	Input:	24 bit floating point number in AEXP, AARG+B0, AARG+B1
;		24 bit floating point number in BEXP, BARG+B0, BARG+B1
;	Use:	CALL FPSWP24

FPSWP24		MOVFW	BEXP
		MOVWF	ACC+B2
		MOVFW	AEXP
		MOVWF	BEXP
		MOVFW	ACC+B2
		MOVWF	AEXP

		MOVFW	BARG+B0
		MOVWF	ACC+B2
		MOVFW	AARG+B0
		MOVWF	BARG+B0
		MOVFW	ACC+B2
		MOVWF	AARG+B0

		MOVFW	BARG+B1
		MOVWF	ACC+B2
		MOVFW	AARG+B1
		MOVWF	BARG+B1
		MOVFW	ACC+B2
		MOVWF	AARG+B1
		return

FPMOV24		movfw	AEXP
		movwf	BEXP
		movfw	AARG+B0
		movwf	BARG+B0
		movfw	AARG+B1
		movwf	BARG+B1
		return
