;
; Subroutine to interface with an LCD display
; Code based on Examples from Peer Ouwehand and Pic'n up the Pace (David Benson)
; **THIS VERSION FOR 4BIT I-FACE, REQUIRES 6 pins !! **
;
; Program	LCD4P.INC
; Author	Wim de Vree, pe1grl@amsat.org
; Revision history
; 2003-03-09 	Created 4 bit version
; 2003-04-04	Bug Fix: no 5ms delay after clear and set home instruction
; 2003-07-27	Added 15 msec delay in LCD_INIT (Power Up delay)
; 2003-10-20	More NOP's added due to problems with 20MHz X-Tal
;		timing was to fast (E-cycle/data setup)
; 2004-02-01	Cosmetic update
; 2004-11-06	LCD startup delay increased to 30 milliseconds, some clone
;		chip like a longer startup delay
;
; Used resources:
;		SCRATCH		< 3 bytes>
;
; Usage:
;		SCRATCH 	equ 0x20	; workspace, 3 bytes required
;						; to be commented
;
;
;*****************************************************************************
; LCD Module Subroutines
;*****************************************************************************
;
 subtitle "LCD34P - Control an LCD using 4bit mode"
;
;  These routines will Initialize, Send Instructions, and Send Characters to a
;   standard Hitachi 44780 controlled LCD.  The display is controlled in 4bit
;  mode requiring a total of 6 lines
;
;  NOTE: 
;	Before the Initialization is to be executed by the PIC, 15 msecs must
;   	elapse before the "Init" subroutine is called.
;	After execution of a command at least 40 uSec is required (160 uSec)
;	CLEAR and HOME require at least 1.6 mSec
;	Enable "high" time should be at least 450 nS (problem with faster PICS)
;
;
; Working registers used
;
LCDTemp		equ	(SCRATCH)	; LCD subroutines internal use
LCDloopv1	equ	(SCRATCH+1)
LCDloopv2	equ	(SCRATCH+2)

;=============================================================================
; LCDINIT
; Initilize LC-Display Module
; Should be modified to your needs (i.e. display type, cursor on/off, etc.)
; OK
;=============================================================================
LCDINIT:				; Initialize the LCD
		movlw	.6		; 30 (6*5) msec startup delay
		movwf	LCDTemp
_lcdinit	call	LCD_Dlay5ms
		decfsz	LCDTemp,F
		 goto	_lcdinit
		bcf	LCD_ENABLE
		bcf	LCD_RS
		call	LCD_Dlay160	; Wait 160 usec
		
	ifdef	LCD_ONE_LINE
		movlw	0x30		; 8-bit, 5*7 mode	1-LINE
	else
		movlw	0x38		; 8-bit, 5*7 mode
	endif
		movwf	LCDTemp
		call	LCD_flipbits
		call	LCD_pulse
		call	LCD_Dlay160	; Wait 160 usec
	ifdef	LCD_ONE_LINE
		movlw	0x20		; 4-bit, 5*7 mode	1-LINE
	else
		movlw	0x28		; 4-bit, 5*7 mode
	endif
		movwf	LCDTemp
		call	LCD_flipbits
		call	LCD_pulse
		call	LCD_Dlay160	; Wait 160 usec
	ifdef	LCD_ONE_LINE
		movlw	0x20		; 4-bit, 5*7 mode	1-LINE
	else
		movlw	0x28		; 4-bit, 5*7 mode
	endif
		call	LCDPUTCMD
		movlw	0x0F
		call	LCDPUTCMD
		movlw	0x01		; Clear the Display, setup the Cursor
		call	LCDPUTCMD
		movlw	0x06		; Set the Entry Mode
		call	LCDPUTCMD
		return

LCDPUTCMD:	movwf	LCDTemp		; Send the Byte as an Instruction
		bcf	LCD_ENABLE
		bcf	LCD_RS		; Start with the R/S Line Low
		call	LCD_flipbits	; Set high nibble
		call	LCD_pulse	; pulse E line
		swapf	LCDTemp,F
		call	LCD_flipbits	; Set low nibble
		call	LCD_pulse	; pulse E line
		call	LCD_Dlay160	; Wait for the LCD to Process the Instruction
		swapf	LCDTemp,W	; original command now in W
		andlw	0xFC		; "Clear Display" and "Cursor At Home" Instructions
		btfsc	STATUS, Z	; Require 5 msec Delay to Complete
		call	LCD_Dlay5ms
		return

LCDPUTCHAR:	movwf	LCDTemp		; Send the Byte as Character Data
		bcf	LCD_ENABLE
		bsf	LCD_RS		; Start with the R/S Line High
		call	LCD_flipbits	; Set high nibble
		call	LCD_pulse	; pulse E line
		swapf	LCDTemp,F
		call	LCD_flipbits	; Set low nibble
		call	LCD_pulse	; pulse E line
		call	LCD_Dlay160	; Wait for the LCD to Process the Instruction
		return

;=============================================================================
; LCDSDDA
; Sets the Display-Data-RAM address. DDRAM data is read/written after
;  this setting.
; Required DDRAM address must be set in W
;  b0-6	: required DDRAM address
;  b7	: don't care
; OK
;=============================================================================
;LCDSDDA		IORLW	0x080		; Function set
;		CALL	LCDPUTCMD
;		RETURN

LCD_flipbits	bcf	LCD_DATA4	; Default '0'
		btfsc	LCDTemp,4	; if '1'
		bsf	LCD_DATA4	; Set to '1'
		bcf	LCD_DATA5
		btfsc	LCDTemp,5
		bsf	LCD_DATA5
		bcf	LCD_DATA6
		btfsc	LCDTemp,6
		bsf	LCD_DATA6
		bcf	LCD_DATA7
		btfsc	LCDTemp,7
		bsf	LCD_DATA7
		return
	
LCD_pulse
    if	CLOCKFREQ > .60000
		nop			; On Fast PICs some extra delay
		nop			; is required (min. 450 nSec)
    endif
		bsf	LCD_ENABLE
		nop
    if	CLOCKFREQ > .60000
		nop			; is required (min. 450 nSec)
		nop
    endif
		bcf	LCD_ENABLE
    if	CLOCKFREQ > .60000
		nop			; On Fast PICs some extra delay
		nop			; is required (min. 450 nSec)
    endif
	return

    if	CLOCKFREQ == .18432			;   1.8432 MHz
	CONSTANT 	LCD160CNT= .15		; constant for 160 us
    else
     if	CLOCKFREQ == .40000 || CLOCKFREQ == .40960 ;  4.000 or 4.096 MHz
	CONSTANT 	LCD160CNT= .52		; constant for 160 us
     else
      if CLOCKFREQ == .200000			; 20.000 MHz
	CONSTANT 	LCD160CNT= .255		; Actually 154 uSec
      else
	ERROR	"LDC4P: No or Invalid Clock frequency defined"
      endif
    endif
   endif
;*****************************************************************************
; Delay_time	= ((DELAY_value * 4) + 4) * Cycle_time
; DELAY_value	= (Delay_time - (3 * Cycle_time)) / (3 * Cycle_time)
;
; i.e. (@ 4MHz crystal)
; Delay_time	= ((32 * 3) + 4) * 1uSec
;		= 100uSec
; DELAY_value	= (500uSec - 3) / 4
;		= 165 (165.33)
;*****************************************************************************

LCD_Dlay160				; 160 microsecond delay
		movlw	LCD160CNT	; +1		1 cycle
		movwf	LCDloopv1	; +2		1 cycle
LCD_dloop160	NOP
		decfsz	LCDloopv1, F	; step 1	1 cycle
		 goto	LCD_dloop160	; step 2	2 cycles
		return			; +3		2 cycles

;*****************************************************************************
LCD_Dlay5ms
		movlw	.31		; Call 160 uSec delay 31 times
		movwf	LCDloopv2
LCD_dloop5ms	call	LCD_Dlay160
		decfsz	LCDloopv2, F
		goto	LCD_dloop5ms
		return
