;*****************************************************************************
;
; Subroutine to interface with an LCD display
; Code based on Peer Ouwehands examples
; **THIS VERSION FOR 8BIT + HANDSHAKE, REQUIRES 11 pins !! **
;
; Program	LCD8P.INC
; Author	Wim de Vree, PE1GRL@PI8WNO	
; Revision history
; 1999-10-27	Created
; 2000-05-07	Added 4.096 Mhz X-tal support, use MACROS.INC
; 2003-04-05	Forgot the BANK macro in one place
;
; Used resources:
;		SCRATCH		< 3 bytes>
;
; Usage:
;		SCRATCH 	equ 0x10	; workspace, 3 bytes required
;						; to be commented
;
;		movlw	0xFA			; Input MSB
;		movwf	(SCRATCH)
;		call	BYT2ASC
;
;
;
    if	CLOCKFREQ == 18432
	CONSTANT 	D500CNT=D'76'	;constant for 500 us
    else
     if	CLOCKFREQ == 40000
	CONSTANT 	D500CNT=D'165'	;constant for 500 us
     else
      if CLOCKFREQ == 40960
	CONSTANT 	D500CNT=D'169'	;constant for 500 us
      else
	ERROR	"LDC8P: No or Invalid Clock frequency defined"
      endif
    endif
   endif

;*****************************************************************************
; Equates used by subroutines
;*****************************************************************************

LCD_DATA	EQU	PORTB		; LCD data lines interface
LCD_DATA_TRIS	EQU	TRISB
LCD_CTRL	EQU	PORTA		; LCD control lines interface

LCD_LINE0	EQU	0x000
LCD_LINE1	EQU	0x040

; PORTA bits
LCD_E		EQU	2		; LCD Enable control line
LCD_RW		EQU	1		; LCD Read/Write control line
LCD_RS		EQU	0		; LCD Register-Select control line

;
; Working registers used
;
LCD_TEMP	equ	(SCRATCH)	; LCD subroutines internal use
DELAY		equ	(SCRATCH+1)	; Used in DELAYxxx routines
X_DELAY		equ	(SCRATCH+2)	; Used in X_DELAYxxx routines

;*****************************************************************************
; LCD Module Subroutines
;*****************************************************************************
;
;=============================================================================
; LCDINIT
; Initilize LC-Display Module
; Should be modified to your needs (i.e. display type, cursor on/off, etc.)
; OK
;=============================================================================
LCDINIT
					; Busy-flag is not yet valid
					; ALL PORT output should output Low.
		BCF	LCD_CTRL, LCD_RS; 
		BCF	LCD_CTRL, LCD_RW; 
		BCF	LCD_CTRL, LCD_E	; 
					; power-up delay
		MOVLW	.30
		CALL	X_DELAY500	; 30 * 0.5mS = 15mS
					; Busy Flag should be valid from here
		MOVLW	0x038		; 8-bit-interface, 2-lines
		CALL	LCDPUTCMD
		MOVLW	0x000		; disp.off, curs.off, no-blink
		CALL	LCDDMODE
		CALL	LCDCLEAR
		MOVLW	0x004		; disp.on, curs.off
		CALL	LCDDMODE
		MOVLW	0x002		; auto-inc (shift-cursor)
		CALL	LCDEMODE
		RETURN
;=============================================================================
; LCD_ENABLE
; Pulses LCD enable pin
; OK
;=============================================================================
LCD_ENABLE
		BSF	LCD_CTRL, LCD_E	; LCD E-line High
		BCF	LCD_CTRL, LCD_E	; LCD E-line Low
		RETURN
;=============================================================================
; LCDBUSY
; Returns when LCD busy-flag is inactive
; OK
;=============================================================================
LCDBUSY
		BANK1			; Select Register page 1
		MOVLW	0x0FF		; Set PORTB for input
		MOVWF	LCD_DATA_TRIS
		BANK0			; Select Register page 0
		BCF	LCD_CTRL, LCD_RS; Set LCD for command mode
		BSF	LCD_CTRL, LCD_RW; Setup to read busy flag
		BSF	LCD_CTRL, LCD_E	; LCD E-line High
		MOVF	LCD_DATA, W	; Read busy flag + DDram address
		BCF	LCD_CTRL, LCD_E	; LCD E-line Low
		ANDLW	0x80		; Check Busy flag, High = Busy
		BTFSS	STATUS, Z
		GOTO	LCDBUSY
LCDNOTBUSY	BCF	LCD_CTRL, LCD_RW
		BANK1			; Select Register page 1
		MOVLW	0x000
		MOVWF	LCD_DATA_TRIS	; Set PORTB for output
		BANK0			; Select Register page 0
		RETURN
;=============================================================================
; LCDCLEAR
; Clears display and returns cursor to home position (upper-left corner).
;
;=============================================================================
LCDCLEAR
		MOVLW	0x001
		CALL	LCDPUTCMD
		RETURN
;=============================================================================
; LCDHOME
; Returns cursor to home position.
; Returns display to original position (when shifted).
;
;=============================================================================
LCDHOME
		MOVLW	0x002
		CALL	LCDPUTCMD
		RETURN
;=============================================================================
; LCDEMODE
; Sets entry mode of display.
; Required entry mode must be set in W
;  b0	: 0 = no display shift	1 = display shift
;  b1	: 0 = auto-decrement	1 = auto-increment
;  b2-7	: don't care
; OK
;=============================================================================
LCDEMODE
		ANDLW	0x003		; Strip upper bits
		IORLW	0x004		; Function set
		CALL	LCDPUTCMD
		RETURN
;=============================================================================
; LCDDMODE
; Sets display control.
; Required display mode must be set in W
;  b0	: 0 = cursor blink off	1 = cursor blink on
;  b1	: 0 = cursor off	1 = cursor on
;  b2	: 0 = display off	1 = display on (display data remains in DDRAM)
;  b3-7	: don't care
; OK
;=============================================================================
LCDDMODE
		ANDLW	0x007		; Strip upper bits
		IORLW	0x008		; Function set
		CALL	LCDPUTCMD
		RETURN
;=============================================================================
; LCDSCGA
; Sets Character-Generator-RAM address. CGRAM is read/written after
;  this setting.
; Required CGRAM address must be set in W
;  b0-5	: required CGRAM address
;  b6-7	: don't care
;
;=============================================================================
LCDSCGA
		ANDLW	0x03F		; Strip upper bits
		IORLW	0x040		; Function set
		CALL	LCDPUTCMD
		RETURN
;=============================================================================
; LCDSDDA
; Sets the Display-Data-RAM address. DDRAM data is read/written after
;  this setting.
; Required DDRAM address must be set in W
;  b0-6	: required DDRAM address
;  b7	: don't care
; OK
;=============================================================================
LCDSDDA
		IORLW	0x080		; Function set
		CALL	LCDPUTCMD
		RETURN
;=============================================================================
; LCDGADDR
; Returns address counter contents, used for both DDRAM and CGRAM.
; RAM address is returned in W
;
;=============================================================================
LCDGADDR
		BANK1			; Select Register page 1
		MOVLW	0x0FF		; Set PORTB for input
		MOVWF	LCD_DATA_TRIS
		BANK0			; Select Register page 0
		BCF	LCD_CTRL, LCD_RS; Set LCD for command mode
		BSF	LCD_CTRL, LCD_RW; Setup to read busy flag
		BSF	LCD_CTRL, LCD_E	; LCD E-line High
		MOVF	LCD_DATA, W	; Read busy flag + RAM address
		BCF	LCD_CTRL, LCD_E	; LCD E-line Low
		ANDLW	0x07F		; Strip upper bit
		BCF	LCD_CTRL, LCD_RW
		BANK1			; Select Register page 1
		MOVLW	0x000
		MOVWF	LCD_DATA_TRIS	; Set PORTB for output
		BANK0			; Select Register page 0
		RETURN
;=============================================================================
; LCDPUTCHAR
; Sends character to LCD
; Required character must be in W
; OK
;=============================================================================
LCDPUTCHAR
		MOVWF	LCD_TEMP	; Character to be sent is in W
		CALL	LCDBUSY		; Wait for LCD to be ready
		BCF	LCD_CTRL, LCD_RW; Set LCD in read mode
		BSF	LCD_CTRL, LCD_RS; Set LCD in data mode
		BSF	LCD_CTRL, LCD_E	; LCD E-line High
		MOVF	LCD_TEMP, W
		MOVWF	LCD_DATA	; Send data to LCD
		BCF	LCD_CTRL, LCD_E	; LCD E-line Low
		RETURN
;=============================================================================
; LCDPUTCMD
; Sends command to LCD
; Required command must be in W
; OK
;=============================================================================
LCDPUTCMD
		MOVWF	LCD_TEMP	; Command to be sent is in W
		CALL	LCDBUSY		; Wait for LCD to be ready
		BCF	LCD_CTRL, LCD_RW; Set LCD in read mode
		BCF	LCD_CTRL, LCD_RS; Set LCD in command mode
		BSF	LCD_CTRL, LCD_E	; LCD E-line High
		MOVF	LCD_TEMP, W
		MOVWF	LCD_DATA	; Send data to LCD
		BCF	LCD_CTRL, LCD_E	; LCD E-line Low
		RETURN


;*****************************************************************************
; Delay_time	= ((DELAY_value * 3) + 4) * Cycle_time
; DELAY_value	= (Delay_time - (4 * Cycle_time)) / (3 * Cycle_time)
;
; i.e. (@ 4MHz crystal)
; Delay_time	= ((32 * 3) + 4) * 1uSec
;		= 100uSec
; DELAY_value	= (500uSec - 4) / 3
;		= 165.33
;		= 165	(1.8 MHz)
;*****************************************************************************
DELAY500	MOVLW	D500CNT		; +1		1 cycle
		MOVWF	DELAY		; +2		1 cycle
DELAY500_LOOP	DECFSZ	DELAY, F	; step 1	1 cycle
		GOTO	DELAY500_LOOP	; step 2	2 cycles
DELAY500_END	RETURN			; +3		2 cycles
;

X_DELAY500	MOVWF	X_DELAY		; +1		1 cycle
X_DELAY500_LOOP	CALL	DELAY500	; step1		wait 500uSec
		DECFSZ	X_DELAY, F	; step2		1 cycle
		GOTO	X_DELAY500_LOOP	; step3		2 cycles
X_DELAY500_END	RETURN			; +2		2 cycles
