;*****************************************************************************
;
; Common subroutines used by all LCD drivers
;
; Currently 3 types of LCD interfaces exists:
; 
; Serial Interface: ( 3 pins) Uses external shift register
; 4Bit Parallel   : ( 6 pins) RW tied to ground, 4 bit mode
; 8Bit Parallel   : (11 pins) Uses R/W to read busy status, can read back
;
; Contains some High-Level commands which are shared by all implementations
;
; File		LCDCOMMON.INC
; Author	Wim de Vree, pe1grl@amsat.org
; Revision history
; 1999-10-27	Created
; 2000-05-07	Added 4.096 Mhz X-tal support, use MACROS.INC
; 2003-03-07	Splitted from LCD8P and LCD3W
; 2003-12-26	Replaced CALL's with GOTO (less code & stack usage, returns over
;		LCDPUTCMD's return instruction)
;
; LCDCLEAR	Clears display and returns cursor to home position (upper-left corner).
; LCDHOME	Returns cursor to home position.
; LCDEMODE	Sets entry mode of display (in W)
; LCDDMODE	Sets display control (in W)
; LCDSCGA	Sets Character-Generator-RAM address (in W)
; LCDSDDA	Sets the Display-Data-RAM address (in W)
;

;=============================================================================
; LCDCLEAR
; Clears display and returns cursor to home position (upper-left corner).
;
;=============================================================================
LCDCLEAR	MOVLW	0x01
		GOTO	LCDPUTCMD

;=============================================================================
; LCDHOME
; Returns cursor to home position.
; Returns display to original position (when shifted).
;
;=============================================================================
LCDHOME		MOVLW	0x02
		GOTO	LCDPUTCMD

;=============================================================================
; LCDEMODE
; Sets entry mode of display.
; Required entry mode must be set in W
;  b0	: 0 = no display shift	1 = display shift
;  b1	: 0 = auto-decrement	1 = auto-increment
;  b2-7	: don't care
; OK
;=============================================================================
LCDEMODE	ANDLW	0x003		; Strip upper bits
		IORLW	0x004		; Function set
		GOTO	LCDPUTCMD

;=============================================================================
; LCDDMODE
; Sets display control.
; Required display mode must be set in W
;  b0	: 0 = cursor blink off	1 = cursor blink on
;  b1	: 0 = cursor off	1 = cursor on
;  b2	: 0 = display off	1 = display on (display data remains in DDRAM)
;  b3-7	: don't care
; OK
;=============================================================================
LCDDMODE	ANDLW	0x007		; Strip upper bits
		IORLW	0x008		; Function set
		GOTO	LCDPUTCMD

;=============================================================================
; LCDSCGA
; Sets Character-Generator-RAM address. CGRAM is read/written after
;  this setting.
; Required CGRAM address must be set in W
;  b0-5	: required CGRAM address
;  b6-7	: don't care
;
;=============================================================================
LCDSCGA		ANDLW	0x03F		; Strip upper bits
		IORLW	0x040		; Function set
		goto	LCDPUTCMD


;=============================================================================
; LCDSDDA
; Sets the Display-Data-RAM address. DDRAM data is read/written after
;  this setting.
; Required DDRAM address must be set in W
;  b0-6	: required DDRAM address
;  b7	: don't care
; OK
;=============================================================================
LCDSDDA		IORLW	0x080		; Function set
		GOTO	LCDPUTCMD
